Exception: Sexp2Ruby::InvalidOption

Inherits:
StandardError
  • Object
show all
Defined in:
lib/sexp2ruby/errors.rb

Overview

Raised when Processor is initialized with an unknown option, or a known option with an invalid value.