Class: Sexp2Ruby::Node::Base
- Inherits:
-
Object
- Object
- Sexp2Ruby::Node::Base
- Defined in:
- lib/sexp2ruby/node/base.rb
Overview
A node in the AST.
Direct Known Subclasses
Alias, And, Arglist, Args, Array, Attrasgn, BackRef, Begin, Block, BlockPass, Break, Call, Case, Cdecl, Class, Colon2, Colon3, Const, Cvar, Cvasgn, Cvdecl, Defined, Defn, Defs, Dot2, Dot3, Dregx, DregxOnce, Dstr, Dsym, Dxstr, Ensure, Evstr, False, Flip2, Flip3, For, Gasgn, Gvar, Hash, Iasgn, If, Iter, Ivar, Kwsplat, Lasgn, Lit, Lvar, Masgn, Match, Match2, Match3, Module, Next, Nil, Not, NthRef, OpAsgn1, OpAsgn2, OpAsgnAnd, OpAsgnOr, Or, Postexe, Redo, Resbody, Rescue, Retry, Return, Sclass, Self, Splat, Str, Super, Svalue, ToAry, True, Undef, Until, Valias, When, While, Xstr, Yield, Zsuper
Constant Summary collapse
- ASSIGN_NODES =
Processor::ASSIGN_NODES
- LF =
Processor::LF
- LINE_LENGTH =
cutoff for one-liners
78
Instance Attribute Summary collapse
-
#processor ⇒ Object
readonly
Returns the value of attribute processor.
Instance Method Summary collapse
-
#initialize(processor) ⇒ Base
constructor
A new instance of Base.
Constructor Details
#initialize(processor) ⇒ Base
Returns a new instance of Base.
32 33 34 |
# File 'lib/sexp2ruby/node/base.rb', line 32 def initialize(processor) @processor = processor end |
Instance Attribute Details
#processor ⇒ Object (readonly)
Returns the value of attribute processor.
10 11 12 |
# File 'lib/sexp2ruby/node/base.rb', line 10 def processor @processor end |